<?php
/*Sauvegarde MySQL*/
if($_SERVER['HTTP_HOST'] != 'localhost') {
$folder='../_db-backups/';
error_reporting(E_ALL);
ini_set('display_errors', true);
class BackupMySQL extends mysqli {
	/** Dossier des fichiers de sauvegardes */
	protected $dossier;
	/** Nom du fichier */
	protected $nom_fichier;
	/** Ressource du fichier GZip */
	protected $gz_fichier;
	/** Constructeur */
	public function __construct($options = array()) { 
		$default = array(
			'host' => ini_get('mysqli.default_host'),
			'username' => ini_get('mysqli.default_user'),
			'passwd' => ini_get('mysqli.default_pw'),
			'dbname' => '',
			'port' => ini_get('mysqli.default_port'),
			'socket' => ini_get('mysqli.default_socket'),
			// autres options
			'dossier' => '../_db-backups/',
			'nbr_fichiers' => 5,
			'nom_fichier' => 'db_'
			);
		$options = array_merge($default, $options);
		extract($options);
		// Connexion de la connexion DB
		@parent::__construct($host, $username, $passwd, $dbname, $port, $socket);
		if($this->connect_error) {
			$this->message('<p>Erreur de connexion (' . $this->connect_errno . ')<br/><br/>
			<a class="button" href="index.php">Tableau de bord</a></p> <p>'. $this->connect_error).'</p>';
			return;
		}
		// Controle du dossier
		$this->dossier = $dossier;
		if(!is_dir($this->dossier)) {
			if (file_exists($folder)) { @mkdir ($folder, 0777); }
			/*$this->message('<p>Erreur le dossier n\'existe pas &quot;' . htmlspecialchars($this->dossier) . '&quot;<br/><br/>
			<a class="button" href="index.php">Tableau de bord</a></p> ');*/
			return;
		}
		// Controle du fichier
		$this->nom_fichier = $nom_fichier.$dbname.'_' . date('dmY') . '.sql.gz';
		$this->gz_fichier = @gzopen($this->dossier . $this->nom_fichier, 'w');
		if(!$this->gz_fichier) {
			$this->message('<p>Erreur de fichier &quot;' . htmlspecialchars($this->nom_fichier) . '&quot;<br/><br/>
			</p> ');
			return;
		}
		// Demarrage du traitement
		$this->sauvegarder();
		$this->purger_fichiers($nbr_fichiers);
	}
	/** Message d'information ( commenter le "echo" pour rendre le script invisible ) */
	protected function message($message = '&nbsp;') {
		echo $message;
	}
	/** Protection des quot SQL */
	protected function insert_clean($string) {
		// Ne pas changer l'ordre du tableau !!!
		$s1 = array( "\\"	, "'"	, "\r", "\n", );
		$s2 = array( "\\\\"	, "''"	, '\r', '\n', );
		return str_replace($s1, $s2, $string);
	}
	/** Sauvegarder les tables */
	protected function sauvegarder() { 
		$this->message('<div id="titreAdmin">Sauvegarde de la base de données</div>');
		$sql  = '--' ."\n";
		$sql .= '-- '. $this->nom_fichier ."\n";
		gzwrite($this->gz_fichier, $sql);
		// Liste les tables
		$result_tables = $this->query('SHOW TABLE STATUS');
		if($result_tables && $result_tables->num_rows) {
			while($obj_table = $result_tables->fetch_object()) {
				//$this->message('- ' . htmlspecialchars($obj_table->{'Name'}));
				// DROP ...
				$sql  = "\n\n";
				$sql .= 'DROP TABLE IF EXISTS `'. $obj_table->{'Name'} .'`' .";\n";
				// CREATE ...
				$result_create = $this->query('SHOW CREATE TABLE `'. $obj_table->{'Name'} .'`');
				if($result_create && $result_create->num_rows) {
					$obj_create = $result_create->fetch_object();
					$sql .= $obj_create->{'Create Table'} .";\n";
					$result_create->free_result();
				}
				// INSERT ...
				$result_insert = $this->query('SELECT * FROM `'. $obj_table->{'Name'} .'`');
				if($result_insert && $result_insert->num_rows) {
					$sql .= "\n";
					while($obj_insert = $result_insert->fetch_object()) {
						$virgule = false;
						$sql .= 'INSERT INTO `'. $obj_table->{'Name'} .'` VALUES (';
						foreach($obj_insert as $val) {
							$sql .= ($virgule ? ',' : '');
							if(is_null($val)) {
								$sql .= 'NULL';
							} else {
								$sql .= '\''. $this->insert_clean($val) . '\'';
							}
							$virgule = true;
						}
						$sql .= ')' .";\n";
					}
					$result_insert->free_result();
				}
				gzwrite($this->gz_fichier, $sql);
			}
			$result_tables->free_result();
		}
		$path='../_db-backups/';
		gzclose($this->gz_fichier);
		$this->message('<div id="contentAdmin"><p>L\'exportation de la base de données est terminée, vous trouverez un fichier dans le dossier <strong>_db-backups</strong>. Vous pouvez aussi télécharger le fichier -> <strong><a style="color:green;" href="'.$path.htmlspecialchars($this->nom_fichier).'">'.htmlspecialchars($this->nom_fichier).'</a></strong>');
	}
	/** Purger les anciens fichiers */
	protected function purger_fichiers($nbr_fichiers_max) {
		//$this->message();
		$this->message('<p>Purge des anciens fichiers...<br/>');
		$fichiers = array();
		// On recupere le nom des fichiers gz
		if($dossier = dir($this->dossier)) {
			while(false !== ($fichier = $dossier->read())) {
				if($fichier != '.' && $fichier != '..') {
					if(is_dir($this->dossier . $fichier)) {
						// Ceci est un dossier ( et non un fichier )
						continue;
					} else {
						// On ne prend que les fichiers se terminant par ".gz"
						if(preg_match('/\.gz$/i', $fichier)) {
							$fichiers[] = $fichier;
						}
					}
				}
			}
			$dossier->close();
		}
		// On supprime les  anciens fichiers
		$nbr_fichiers_total = count($fichiers);
		if($nbr_fichiers_total >= $nbr_fichiers_max) {
			// Inverser l'ordre des fichiers gz pour ne pas supprimer les derniers fichiers
			rsort($fichiers);
			// Suppression...
			for($i = $nbr_fichiers_max; $i < $nbr_fichiers_total; $i++) {
				$this->message('Base supprimée -> <strong style="color:red;">' . htmlspecialchars($fichiers[$i]) . '</strong><br/>');
				unlink($this->dossier . $fichiers[$i]);
			}
		}
		$this->message('Purge terminée !<br/><br/><br/>
		</p></div>');
	}
}
//nom de l'hote, nom utilisateur mysql, mot de passe mysql, nom de la bdd
if($_SERVER['HTTP_HOST'] != 'localhost') {
new BackupMySQL(array(
	'host' => $sqlserver,
	'username' => $sqluser,
	'passwd' => $sqlpass,
	'dbname' => $bdd
));
} else {
new BackupMySQL(array(
	'host' => 'localhost',
	'username' => $_SESSION['login'],
	'passwd' => '',
	'dbname' => $bdd
));
}
} else {
	echo '<div id="titreAdmin">Sauvegarde de la base de données<p>La sauvegarde n\'est pas active en mode local</p></div>
	<div id="contentAdmin"><p>Pour des raisons techniques et pratiques, la sauvegarde n\'est pas activée en local mais </strong>seulement en ligne</strong> chez l\'hébergeur.
	<br/>Vous pouvez toutefois la tester sur le site de <a href="http://gdacad.free.fr/_admin"><strong>démonstration en ligne</strong></a> du CMS, sinon continuer vers d\'autres fonctionnalités du CMS.</p>
	</div>';
}
?>
